94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
91 lines
4.0 KiB
Plaintext
91 lines
4.0 KiB
Plaintext
#pragma once
|
|
|
|
#include "hicache.cuh"
|
|
#include <limits>
|
|
|
|
namespace {
|
|
|
|
struct HicacheRelayoutParams {
|
|
void* __restrict__ k_cache_dst;
|
|
void* __restrict__ v_cache_dst;
|
|
const void* __restrict__ indices_src;
|
|
const void* __restrict__ k_ptr_src;
|
|
const void* __restrict__ v_ptr_src;
|
|
uint32_t num_pages;
|
|
uint32_t num_layers;
|
|
uint32_t page_size;
|
|
};
|
|
|
|
template <typename IndexType, int64_t kElementSize, bool kIsMLA>
|
|
__global__ void hicache_relayout_kernel(const __grid_constant__ HicacheRelayoutParams params) {
|
|
using namespace device;
|
|
using pack_t = uint4;
|
|
static_assert(kElementSize % 16 == 0, "hicache_relayout_kernel requires 16-byte aligned element size");
|
|
constexpr uint32_t kVecBytes = 16;
|
|
constexpr uint32_t kVecPerItem = kElementSize / kVecBytes;
|
|
|
|
const auto& [k_cache_dst, v_cache_dst, indices_src, k_ptr_src, v_ptr_src, num_pages, num_layers, page_size] = params;
|
|
const auto k_ptr_src_arr = static_cast<const void* const*>(k_ptr_src);
|
|
const auto v_ptr_src_arr = static_cast<const void* const*>(v_ptr_src);
|
|
const auto tid = static_cast<uint64_t>(blockIdx.x) * blockDim.x + threadIdx.x;
|
|
const auto stride = static_cast<uint64_t>(gridDim.x) * blockDim.x;
|
|
const auto total_vecs = static_cast<uint64_t>(num_pages) * page_size * num_layers * kVecPerItem;
|
|
|
|
for (uint64_t linear_vec_id = tid; linear_vec_id < total_vecs; linear_vec_id += stride) {
|
|
const auto page_id =
|
|
static_cast<uint32_t>(linear_vec_id / (static_cast<uint64_t>(page_size) * num_layers * kVecPerItem));
|
|
const auto page_vec_id =
|
|
static_cast<uint32_t>(linear_vec_id % (static_cast<uint64_t>(page_size) * num_layers * kVecPerItem));
|
|
const auto token_in_page = page_vec_id / (num_layers * kVecPerItem);
|
|
const auto token_vec_id = page_vec_id % (num_layers * kVecPerItem);
|
|
const auto layer_id = token_vec_id / kVecPerItem;
|
|
const auto vec_id = token_vec_id % kVecPerItem;
|
|
const auto src_page = static_cast<uint32_t>(static_cast<const IndexType*>(indices_src)[page_id]);
|
|
const auto src_token = src_page + token_in_page;
|
|
const auto src_k = pointer::offset(
|
|
static_cast<const void*>(k_ptr_src_arr[layer_id]),
|
|
static_cast<int64_t>(src_token) * kElementSize + static_cast<int64_t>(vec_id) * kVecBytes);
|
|
const auto dst_k =
|
|
pointer::offset(static_cast<void*>(k_cache_dst), static_cast<int64_t>(linear_vec_id) * kVecBytes);
|
|
const auto vec_k = details::load_nc(reinterpret_cast<const pack_t*>(src_k));
|
|
details::store_nc(reinterpret_cast<pack_t*>(dst_k), vec_k);
|
|
|
|
if constexpr (!kIsMLA) {
|
|
const auto src_v = pointer::offset(
|
|
static_cast<const void*>(v_ptr_src_arr[layer_id]),
|
|
static_cast<int64_t>(src_token) * kElementSize + static_cast<int64_t>(vec_id) * kVecBytes);
|
|
const auto dst_v =
|
|
pointer::offset(static_cast<void*>(v_cache_dst), static_cast<int64_t>(linear_vec_id) * kVecBytes);
|
|
const auto vec_v = details::load_nc(reinterpret_cast<const pack_t*>(src_v));
|
|
details::store_nc(reinterpret_cast<pack_t*>(dst_v), vec_v);
|
|
}
|
|
}
|
|
}
|
|
|
|
template <int64_t kElementSize, bool kIsMLA>
|
|
inline void launch_hicache_relayout_kernel(
|
|
const HicacheRelayoutParams& params,
|
|
int64_t num_pages,
|
|
int64_t num_layers,
|
|
int64_t page_size,
|
|
bool use_int32,
|
|
DLDevice device) {
|
|
using namespace host;
|
|
|
|
constexpr uint32_t kRelayoutBlockSize = 256;
|
|
constexpr uint32_t kVecPerItem = kElementSize / 16;
|
|
const auto total_vecs = static_cast<uint64_t>(num_pages) * page_size * num_layers * kVecPerItem;
|
|
const auto kernel = use_int32 ? hicache_relayout_kernel<int32_t, kElementSize, kIsMLA>
|
|
: hicache_relayout_kernel<int64_t, kElementSize, kIsMLA>;
|
|
if (total_vecs == 0) {
|
|
return;
|
|
}
|
|
|
|
const auto grid = div_ceil(total_vecs, static_cast<uint64_t>(kRelayoutBlockSize));
|
|
RuntimeCheck(
|
|
grid <= std::numeric_limits<uint32_t>::max(), "HiCache staged relayout: CUDA grid size exceeds uint32 range");
|
|
LaunchKernel(static_cast<uint32_t>(grid), kRelayoutBlockSize, device)(kernel, params);
|
|
}
|
|
|
|
} // namespace
|